import { PRE_GAME, PRE_ROUND, IN_ROUND, POST_ROUND, POST_GAME } from "./GameState";
import { createStore } from "../store";
import { joinGame, sendGuess, getCurrentRound } from "./apiMethods";
import {
  saveGameInfoToLocalStorage,
  clearGameInfoFromLocalStorage,
  getGameInfoFromLocalStorage,
  saveTimerToLocalStorage,
  clearRoundInfoFromLocalStorage,
  getRoundInfoFromLocalStorage,
} from "./localStorageMethods";

export const [
  {
    useGameId,
    usePlayerName,
    useLastRound,
    usePlayerId,
    useGameState,
    useCurrentRound,
    useTargetPoint,
    useRoundSeconds,
    usePanoStartPosition,
    usePanoStartPov,
  },
  dispatch,
] = createStore({
  gameId: null,
  playerName: null,
  lastRound: {
    roundNum: -1,
    targetPoint: {
      lat: 0,
      lng: 0,
    },
    score: -1,
    totalScore: -1,
  },
  playerId: null,
  gameState: PRE_GAME,
  currentRound: null,
  targetPoint: null,
  roundSeconds: 0,
  panoStartPosition: null,
  panoStartPov: null,
}, {
  setPlayerName: ([set], playerName) => set({ playerName }),
  goToLobby: ([set], gameId) => set({ 
    gameId,
    playerId: null,
    gameState: PRE_ROUND,
  }),
  joinGame: async ([set, get]) => {
    const gameId = get.gameId();
    const name = get.playerName();
    const { playerId } = await joinGame(gameId, name);
    // TODO pull this chunk out
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    set({
      playerId,
      currentRound,
      targetPoint: coord,
      panoStartPosition: coord,
      panoStartPov: { heading: 0, pitch: 0 },
      roundSeconds: timer,
    });
    saveGameInfoToLocalStorage(gameId, name, playerId);
  },
  rejoinGame: async ([set]) => {
    const { gameId, playerName, playerId } = getGameInfoFromLocalStorage();
    set({ gameId, playerName, playerId });
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    const { savedPosition, savedPov, savedTimer } = getRoundInfoFromLocalStorage();
    set({
      currentRound,
      targetPoint: coord,
      roundSeconds: savedTimer ?? timer,
      panoStartPosition: savedPosition ?? coord,
      panoStartPov: savedPov ?? { heading: 0, pitch: 0 },
      gameState: IN_ROUND,
    });
  },
  startRound: ([set]) => set({ gameState: IN_ROUND }),
  submitGuess: async ([set, get], selectedPoint) => {
    clearRoundInfoFromLocalStorage();
    const gameId = get.gameId();
    const playerId = get.playerId();
    const roundNum = get.currentRound();
    const targetPoint = get.targetPoint();
    const { score, totalScore } = await sendGuess(
      gameId,
      playerId,
      roundNum,
      selectedPoint || { timeout: true }
    );
    set({
      lastRound: {
        roundNum,
        targetPoint,
        score,
        totalScore,
      },
      gameState: POST_ROUND,
    });
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    set({
      currentRound,
      targetPoint: coord,
      panoStartPosition: coord,
      panoStartPov: { heading: 0, pitch: 0 },
      roundSeconds: timer,
    });
  },
  goToSummary: ([set], gameId, clearSavedGame = true) => {
    if (gameId) {
      set({ gameId });
    }
    set({ gameState: POST_GAME });
    if (clearSavedGame) {
      clearRoundInfoFromLocalStorage();
      clearGameInfoFromLocalStorage();
    }
  },
  setRoundSeconds: ([set], roundSeconds) => {
    set({ roundSeconds });
    saveTimerToLocalStorage(roundSeconds);
  },
});